import json
import os
import shutil
from selenium import webdriver
from uitestcore.utilities.config_handler import parse_config_data
from uitestcore.utilities.datetime_handler import get_current_datetime
from uitestcore.utilities.string_util import remove_invalid_characters

SCREENSHOTS_PATH = "screenshots"


class BrowserHandler:
    """
    This class performs any necessary tasks related to the browser
    """

    @staticmethod
    def set_browser_size(context):
        """
        Sets the browser window to the required size e.g. maximized
        :param context: the test context instance
        """
        if context.maximize_browser:
            context.browser.maximize_window()

    @staticmethod
    def prepare_browser(context):
        """
        Set up the browser based on the config options
        :param context: the test context instance
        """
        # Check if we have any command line parameters to parse
        parse_config_data(context)

        # Check the Browser specified in config and load the Selenium Web Driver
        open_browser(context)

        # Set Implicit Wait on Selenium Driver
        context.browser.implicitly_wait(context.implicit_wait)

        # Check if Maximize Browser Flag has been activated
        BrowserHandler.set_browser_size(context)

    @staticmethod
    def take_screenshot(driver, description):
        """
        Save a screenshot of the browser window - should be used after a test fails
        :param driver: the browser driver
        :param description: information about the screenshot to be added to the file name
        :return: boolean representing whether saving the screenshot succeeded
        """
        window_width = driver.get_window_size()["width"]
        window_height = driver.get_window_size()["height"]
        scroll_height = driver.execute_script("return document.body.scrollHeight")

        # Set the browser to the full height of the page so that everything is captured
        if scroll_height > window_height:
            driver.set_window_size(window_width, scroll_height)

        # Create the screenshots folder
        if not os.path.exists(SCREENSHOTS_PATH):
            os.makedirs(SCREENSHOTS_PATH)

        # Create a file name and ensure it is not too long
        timestamp = get_current_datetime().strftime("%Y-%m-%d_%H.%M.%S.%f")
        description = remove_invalid_characters(description)
        file_name = f"{SCREENSHOTS_PATH}/{timestamp}_{description}"
        file_name = (file_name[:100] + "---.png") if len(file_name) > 100 else file_name + ".png"

        # Save the screenshot
        result = driver.save_screenshot(file_name)

        # Reset the browser size if it was changed
        if scroll_height > window_height:
            driver.set_window_size(window_width, window_height)

        return result

    @staticmethod
    def move_screenshots_to_folder(folder_name):
        """
        Create a new folder and move all screenshots from the root screenshot folder into it (to be run at end of test)
        :param folder_name: the name of the folder into which the screenshots should be moved
        """
        folder_name = remove_invalid_characters(folder_name)
        source = f"{SCREENSHOTS_PATH}/"
        destination = source + folder_name
        files = os.listdir(source)

        if not os.path.exists(destination):
            os.makedirs(destination)

        for file in files:
            if file.endswith(".png"):
                shutil.move(source + file, destination)


def open_browser(context):
    """
    Open the required browser
    :param context: the test context instance
    """
    if context.browser.lower() == "chrome":
        open_chrome(context)

    elif context.browser.lower() == "firefox":
        open_firefox(context)

    elif context.browser.lower() == "browserstack":
        start_browserstack(context)

    else:
        raise ValueError(f"Browser '{context.browser}' not supported")


def open_chrome(context):
    """
    Open the Chrome browser
    :param context: the test context instance
    """
    if os.name == 'nt':
        context.browser = webdriver.Chrome(executable_path=r"./browser_executables/chromedriver.exe")

    else:
        chrome_options = webdriver.ChromeOptions()
        chrome_options.add_argument("--no-sandbox")
        chrome_options.add_argument("--window-size=1420,1080")
        chrome_options.add_argument("--headless")
        chrome_options.add_argument("--disable-gpu")

        # No need to specify the executable as we're using one installed via pip in Dockerfile
        context.browser = webdriver.Chrome(chrome_options=chrome_options)

    BrowserHandler.set_browser_size(context)


def open_firefox(context):
    """
    Open the Firefox browser
    :param context: the test context instance
    """
    if os.name == 'nt':
        context.browser = webdriver.Firefox(executable_path=r"browser_executables/geckodriver.exe")

    else:
        firefox_options = webdriver.FirefoxOptions()
        firefox_options.add_argument("--headless")

        # The Firefox driver (geckodriver) must be located in the "firefox" folder when running in Docker
        context.browser = webdriver.Firefox(executable_path=r"firefox/geckodriver", firefox_options=firefox_options)

    BrowserHandler.set_browser_size(context)


def start_browserstack(context):
    """
    Start Browserstack with the required options
    :param context: the test context instance
    """
    config_file = os.environ['CONFIG_FILE'] if 'CONFIG_FILE' in os.environ else 'config/BrowserStackConfig.json'
    task_id = int(os.environ['TASK_ID']) if 'TASK_ID' in os.environ else 0

    with open(config_file) as data_file:
        config = json.load(data_file)

    browserstack_username = os.environ['BROWSERSTACK_USERNAME']
    browserstack_access_key = os.environ['BROWSERSTACK_ACCESS_KEY']
    desired_capabilities = config['environments'][task_id]
    for key in config["capabilities"]:
        if key not in desired_capabilities:
            desired_capabilities[key] = config["capabilities"][key]
    context.browser = webdriver.Remote(
        desired_capabilities=desired_capabilities,
        command_executor="http://%s:%s@%s/wd/hub" % (browserstack_username, browserstack_access_key, config['server'])
    )



"""
You can create your own logger and pass it into the uitestcore
However if you don't want to, and just want something simple, you might find this useful
"""
import logging
import logging.config
import os
import string
import time

from uitestcore.utilities.string_util import generate_random_string


def init_unique_log_file_logger(file_path=os.path.abspath('logs'),
                                level=logging.INFO,
                                log_format='%(asctime)s - %(levelname)s - %(name)s: %(message)s'):
    """
    Setting up the basic config for the logging module. Creates a log file in the specified location.
    File name will be time.strftime('%Y%m%d-%H%M%S') + '.log'
    You can specify a logging level and the format of the log messages

    :param file_path: default os.path.abspath('logs')
    :param level: default logging.INFO
    :param log_format: default '%(asctime)s - %(levelname)s - %(name)s: %(message)s'
    """
    if not os.path.exists(file_path):
        os.makedirs(file_path)

    for handler in logging.root.handlers[:]:
        logging.root.removeHandler(handler)

    file_name = os.path.join(file_path, time.strftime('%Y%m%d-%H%M%S') + '.log')
    logging.basicConfig(filename=file_name, filemode='w', level=level, format=log_format)
    logging.debug("initialised logger to write to file %s with level %s and format %s",
                  file_name, level, log_format)


def auto_log(logger_name):
    """
    A decorator that wraps the passed in function, logs entering and exiting the function,
    and logs exceptions should one occur. Exceptions are reraised.
    Tag each method or function with @auto_log(__name__) to automatically log calls.
    Logs these calls at DEBUG level.

    :param logger_name: The class or module name of the calling method. Use __name__
    """

    def decorator(func):
        def wrapper(*args, **kwargs):
            logger = logging.getLogger(logger_name)
            log_identifier = generate_random_string(6, chars=string.digits)
            try:
                entry_message = "Entering function %s (Function call ID %s) " \
                    "with args %s and kwargs %s"
                logger.debug(entry_message, func.__name__, log_identifier, args, kwargs)
                return_value = func(*args, **kwargs)
                exit_message = "Exited function %s (Function call ID %s) " \
                    "with return value %s"
                logger.debug(exit_message, func.__name__, log_identifier, return_value)
                return return_value
            except Exception as e:
                # log the exception
                err = "There was an exception thrown in function %s (Function call ID %s)"
                logger.exception(err, func.__name__, log_identifier)

                # re-raise the exception
                raise e

        return wrapper
    return decorator



import logging
from selenium.webdriver.remote.webelement import WebElement
from selenium.webdriver.support.select import Select

from uitestcore.utilities.logger_handler import auto_log


class Interactor:
    """
    Interact with elements without returning a value, such as clicking, sending keys, or performing Actions
    If you're expecting a return value, such as the Element or True/False look in the Finder or Interrogator
    """

    def __init__(self, driver, finder, interrogator, waiter, existing_logger=None):
        """
        Default constructor which passes the control of webDriver to the current page
        :param driver: the Selenium web driver
        :param finder: Finder
        :param interrogator: Interrogator
        :param waiter: Waiter
        :param existing_logger: logger object used to save information to a log file
        """
        self.driver = driver
        self.find = finder
        self.interrogate = interrogator
        self.wait = waiter
        self.logger = existing_logger or logging.getLogger(__name__)

    @auto_log(__name__)
    def click_element(self, page_element):
        """
        Finds and clicks on an element on the page
        :param page_element: PageElement instance representing the element
        """
        self.find.element(page_element).click()

    @auto_log(__name__)
    def execute_click_with_java_script(self, page_element):
        """
        execute the click on the element using javascript.
        :param page_element: the element to click
        :return: element clicked
        """
        element: WebElement = self.find.element(page_element)
        return self.driver.execute_script("arguments[0].click();", element)

    @auto_log(__name__)
    def select_by_visible_text(self, page_element, visible_text_to_select):
        """
        Select all options that display text matching the visible_text_to_select argument.
        :param page_element: the element to select
        :param visible_text_to_select: The visible text to select in the drop down
        """
        element: WebElement = self.find.element(page_element)
        Select(element).select_by_visible_text(visible_text_to_select)

    @auto_log(__name__)
    def select_by_value(self, page_element, value):
        """
        Select all options that have a value matching the argument
        :param page_element: the element to select
        :param value:The value to match against
        """
        element: WebElement = self.find.element(page_element)
        Select(element).select_by_value(value)

    @auto_log(__name__)
    def select_by_index(self, page_element, index):
        """
        Select the option at the given index. This is done by examining the "index" attribute of an
        element
        :param page_element:the element to select
        :param index:The option at this index will be selected
        """
        element: WebElement = self.find.element(page_element)
        Select(element).select_by_index(index)

    @auto_log(__name__)
    def enter_text(self, page_element, field_input, clear_first=True):
        """
        Writes the given text to an element on the page - only to be used with editable text fields
        :param page_element: PageElement instance representing the element
        :param field_input: the text to write to the element
        :param clear_first: boolean representing whether or not to clear the field before editing (default True)
        """
        element: WebElement = self.find.element(page_element)
        if clear_first:
            element.clear()
        element.send_keys(field_input)

    @auto_log(__name__)
    def send_keys(self, page_element, key):
        """
        Send a Key action to an element
        :param page_element: PageElement instance representing the element
        :param key: the key to press e.g. Keys.ARROW_DOWN
        """
        self.enter_text(page_element, key, False)

    @auto_log(__name__)
    def open_url(self, url):
        """
        open any given url
        :param url:
        :return: None
        """
        self.driver.get(url)
        self.wait.for_page_to_load()
        self.logger.info("Navigated to the URL - %s", url)

    @auto_log(__name__)
    def append_and_open_url(self, additional_url):
        """
        Adds additional data to URL e.g. query string
        :param additional_url: the string to append the URL
        """
        url = self.interrogate.get_current_url()
        url += additional_url
        self.open_url(url)

    @auto_log(__name__)
    def close_current_window(self):
        """
        Close the current window/tab
        Then will switch to a remaining window if available
        :return: None
        """
        remaining_windows = self.driver.window_handles
        remaining_windows.remove(self.driver.current_window_handle)
        self.driver.close()
        if remaining_windows:
            self.driver.switch_to_window(remaining_windows[len(remaining_windows)-1])

    @auto_log(__name__)
    def scroll_into_view(self, page_element):
        """
        scroll an element into view using javascript
        :param page_element:
        :return:
        """
        self.logger.info("Scrolling to %s", page_element)
        element: WebElement = self.find.element(page_element)
        self.driver.execute_script("arguments[0].scrollIntoView();", element)

    @auto_log(__name__)
    def switch_to_next_window(self):
        """
        Switch the control to a new page that opens up
        :return: None
        """
        new_window = self.driver.window_handles[len(self.driver.window_handles) - 1]
        self.logger.info("Switching to next window")
        self.driver.switch_to_window(new_window)
        self.logger.info("Switched to next window")

    @auto_log(__name__)
    def switch_to_original_window(self):
        """
        Switch the control to the original window with a window_handle index of 0
        :return:None
        """
        old_window = self.driver.window_handles[0]
        self.driver.switch_to_window(old_window)

    @auto_log(__name__)
    def switch_to_frame(self, page_element):
        """
        Switch the control into an iframe
        :param page_element: iframe
        :return: None
        """
        self.driver.switch_to.frame(self.find.element(page_element))

    @auto_log(__name__)
    def accept_alert(self):
        """
        Accept an alert using built in selenium
        :return:
        """
        self.driver.switch_to.alert.accept()

    @auto_log(__name__)
    def dismiss_alert(self):
        """
        Dismisses an alert using built in selenium
        :return: None
        """
        self.driver.switch_to.alert.dismiss()

    @auto_log(__name__)
    def enter_text_into_alert(self, text):
        """
        Sends keys to the alert using Selenium
        :param text:
        :return: None
        """
        self.driver.switch_to.alert.send_keys(text)

    @auto_log(__name__)
    def switch_to_default_content(self):
        """
        Switch focus to the default frame
        """
        self.driver.switch_to.default_content()

    @auto_log(__name__)
    def clear_cookie_and_refresh_page(self, cookie_name):
        """
        Delete a single cookie and refresh the related page. The name of the cookie must be supplied and matched.
        :param cookie_name: The name of the cookie to search for and to delete
        """
        self.driver.delete_cookie(cookie_name)
        self.driver.refresh()

    @auto_log(__name__)
    def clear_all_cookies(self):
        """
        Delete all cookies for the current page - this does not refresh the page
        """
        self.driver.delete_all_cookies()



"""
A useful collection of date and time utility methods
"""
import datetime
from dateutil.relativedelta import relativedelta


def get_current_date():
    """
    Uses the standard datetime lib to return date.today()
    :return: Date in YYY-MM-DD format
    """
    return datetime.date.today()


def get_current_datetime():
    """
    Uses the standard datetime lib to return date.now()
    :return: Date in YYY-MM-DD-HH-MM-SS format
    """
    return datetime.datetime.now()


def get_date_altered_by_days(day_difference, current_date=datetime.date.today()):
    """
    Generates a date offset by a number of days
    Can be a date in the future or in past, by passing in a positive or negative day_difference
    :param day_difference: int number of days to offset from current date
    :param current_date: optional. can be used to set a different starting date to offset from. Used for unit tests
    :return: Date in YYY-MM-DD format
    """
    return current_date + relativedelta(days=day_difference)


def get_date_altered_by_months(month_difference, current_date=datetime.date.today()):
    """
    Generates a date offset by a number of days
    Can be a date in the future or in past, by passing in a positive or negative month_difference
    :param month_difference: int number of months to offset from current date
    :param current_date: optional. can be used to set a different starting date to offset from. Used for unit tests
    :return: Date in YYY-MM-DD format
    """
    return current_date + relativedelta(months=month_difference)



from enum import Enum


class PageElement:
    """
    This represents an element on the page with information which can be used to locate it
    """

    def __init__(self, locator_type, locator_value, element_type=""):
        """
        Default constructor to create a PageElement
        :param locator_type: the type of Selenium locator e.g. By.ID
        :param locator_value: the locator string e.g. the ID of the element
        :param element_type: additional information about an element e.g. "text" represents a text field
        """
        self.locator_type = locator_type
        self.locator_value = locator_value
        self.field_type = element_type

    def __add__(self, value):
        """
        Overrides the "+" operator for this class to allow appending of locator value
        :param value: the string to append to the locator value
        :return: a new PageElement instance with the altered locator value
        """
        return PageElement(self.locator_type, self.locator_value + value)

    def __str__(self):
        """
        Provides a string representation of the PageElement
        :return: String
        """
        return f"PageElement {self.locator_type}='{self.locator_value}'"


class FieldTypes(Enum):
    text_box = "text"
    radio_button = "radio"
    check_box = "check"



def parse_config_data(context):
    """
    Set any relevant config items on the context
    :param context: the test context instance
    """
    if context.config.userdata:

        for key, value in context.config.userdata.items():

            if key.lower() == 'base_url':
                context.url = value
                continue

            elif key.lower() == "logging_flag":

                if value.lower() == "false":
                    context.logging_flag = False
                    continue

                else:
                    context.logging_flag = True
                    continue

            elif key.lower() == "maximize_browser_flag":

                if value.lower() == "false":
                    context.maximize_browser = False
                    continue

                else:
                    context.maximize_browser = True
                    continue

            elif key.lower() == "browser":
                context.browser = value

    else:
        print("No Command line Params detected, using Config file values")



import logging

from uitestcore.utilities.logger_handler import auto_log


class Finder:
    """
    Selenium based find methods
    Use this class to first find an element(s) before interacting with it
    """

    def __init__(self, driver, existing_logger=None):
        """
        Default constructor which passes the control of webDriver to the current page
        :param driver: the Selenium web driver
        :param existing_logger: logger object used to save information to a log file
        """
        self.driver = driver
        self.logger = existing_logger or logging.getLogger(__name__)

    @auto_log(__name__)
    def elements(self, page_element):
        """
        Find the elements matching the given page element object
        :param page_element: PageElement instance representing the element
        :return: list of matching WebElements
        """
        return self.driver.find_elements(page_element.locator_type, page_element.locator_value)

    @auto_log(__name__)
    def element(self, page_element):
        """
        Find a single element matching the given page element object
        If no element found, will return None
        :param page_element: PageElement instance representing the element
        :return: single WebElement or None
        """
        try:
            return self.elements(page_element)[0]
        except IndexError:
            return None

    @auto_log(__name__)
    def visible_elements(self, page_element):
        """
        Find the elements matching the given page element object, only returning the visible ones
        :param page_element: PageElement instance representing the element
        :return: list of matching WebElements which are visible
        """
        elements = self.driver.find_elements(page_element.locator_type, page_element.locator_value)
        visible_elements = []

        for element in elements:
            if element.is_displayed() and not element.get_attribute("aria-hidden") == "true":
                visible_elements.append(element)

        return visible_elements

    @auto_log(__name__)
    def number_of_elements(self, page_element):
        """
        Count the number of matching elements on the page
        :param page_element: PageElement instance representing the element
        :return: the number of elements found
        """
        elements = self.elements(page_element)
        return len(elements)



"""
This module handles attaching files to test result records on Azure DevOps
Uses Azure DevOps Services REST API:
https://docs.microsoft.com/en-us/rest/api/azure/devops/test/attachments?view=azure-devops-rest-5.0
"""

import base64
import datetime
import json
from os import listdir
from xml.etree.ElementTree import fromstring
import requests
from uitestcore.utilities.string_util import remove_invalid_characters

AZURE_API_VERSION_GET = "5.0"
AZURE_API_VERSION_POST = "5.0-preview.1"


def attach_files(organisation, project, attachment_file_path, args):
    """
    Get the details of the failed tests from a given release and attach the files using Microsoft API calls
    The organisation and project names can be found in the project URL e.g. dev.azure.com/organisation/project
    :param organisation: the Azure organisation name
    :param project: the Azure project name
    :param attachment_file_path: the file path to the directory containing the files to attach
    :param args: sys.argv which should contain the release ID and auth token
    :return: integer: a return value of 0 indicates success, 1 means that any file could not be attached
    """
    # Get the release ID and auth token from the passed arguments
    params = parse_parameters(args)
    if not params:
        return 1
    release_id = params[0]
    auth_token = params[1]

    # Set the URL for the required API
    request_url = f"https://dev.azure.com/{organisation}/{project}/_apis/test/runs"

    # Get the run IDs for the given release
    run_ids = get_run_ids(release_id, request_url, auth_token)

    if not run_ids:
        print_azure_error(f"No test runs found for release {release_id}")
        return 1

    print(f"Run IDs found for release {release_id}: {run_ids}")

    # Get the list of failed tests
    failed_tests = get_failed_tests(run_ids, request_url, auth_token)

    if not failed_tests:
        return 1

    print(f"Failed tests found (run ID, test case result ID, test name): {failed_tests}")

    # A return value of 0 indicates success, 1 means that any file could not be attached
    return_value = 0

    # Ensure that the file path ends with a "/"
    if not attachment_file_path.endswith("/"):
        attachment_file_path += "/"

    # Attach the relevant files
    for failed_test in failed_tests:
        # Get all of the file names to attach for this test
        file_path = attachment_file_path + remove_invalid_characters(failed_test[2])
        file_names = []

        if append_file_names(file_names, file_path) == 1:
            return_value = 1

        if not file_names:
            continue

        # Attach any files found for this test
        for file_name in file_names:
            file_b64 = get_file_base64(f"{file_path}/{file_name}")

            if not file_b64:
                print_azure_error(f"Could not convert file to Base64: {file_path}/{file_name}")
                return_value = 1
                continue

            run_id = failed_test[0]
            test_case_result_id = failed_test[1]

            response = requests.post(
                f"{request_url}/{run_id}/Results/{test_case_result_id}/attachments",
                params={"api-version": AZURE_API_VERSION_POST},
                auth=("", auth_token),
                headers={"Content-Type": "application/json"},
                data=json.dumps({
                    "attachmentType": "GeneralAttachment",
                    "comment": "Attached by UiTestCore",
                    "fileName": file_name,
                    "stream": file_b64.decode("utf-8")
                })
            )

            print(f"Attach file {file_name} - response {response.status_code}")

            if not response.status_code == 200:
                return_value = 1

    return return_value


def parse_parameters(args):
    """
    Parse the release ID and auth token from the provided arguments
    :param args: sys.argv which should contain the release ID and auth token (args[0] is not used)
    :return: tuple containing the two parameters or None if there was an error
    """
    try:
        params = args[1], args[2]

    except IndexError:
        print_azure_error("Unable to parse required parameters - ensure they are passed correctly. "
                          "Expected release ID and auth token.")
        return None

    else:
        print(f"The release ID is: {params[0]}")
        return params


def get_run_ids(release_id, request_url, auth_token):
    """
    Get the test run IDs for the given release ID - each feature will have a unique run ID
    :param release_id: the release ID from Azure DevOps
    :param request_url: the URL for the Microsoft API
    :param auth_token: the ID number of the release from Azure DevOps
    :return: list of run IDs which were found
    """
    max_last_updated_date = datetime.datetime.now()
    min_last_updated_date = max_last_updated_date - datetime.timedelta(days=1)

    response = requests.get(
        request_url,
        params={"minLastUpdatedDate": min_last_updated_date,
                "maxLastUpdatedDate": max_last_updated_date,
                "releaseIds": release_id,
                "api-version": AZURE_API_VERSION_GET
                },
        auth=("", auth_token)
    )

    print(f"Get run IDs for release {release_id} - response {response.status_code}")

    if not response.status_code == 200:
        print_azure_error(f"Could not get run IDs for release {release_id}")
        print_response_info(response)
        return []

    response_json = response.json()
    run_ids = []

    for item in response_json["value"]:
        run_ids.append(item["id"])

    return run_ids


def print_response_info(response):
    """
    Prints additional infomration from the title of the response e.g. to show when the token has expired
    :param response: the response object returned by the request
    """
    if response.content:
        info = fromstring(response.content).findtext('.//title')
        if info:
            print(info)


def get_failed_tests(run_ids, request_url, auth_token):
    """
    Get the required details of the failed tests from the given runs
    :param run_ids: list of run IDs to query
    :param request_url: the URL for the Microsoft API
    :param auth_token: the ID number of the release from Azure DevOps
    :return: list of test details in the format: (run ID, test ID, test name)
    """
    failed_tests = []

    for run_id in run_ids:
        response = requests.get(
            request_url + f"/{run_id}/results",
            params={"api-version": AZURE_API_VERSION_GET},
            auth=("", auth_token)
        )

        print(f"Get failed tests for run ID {run_id} - response {response.status_code}")

        if not response.status_code == 200:
            print_azure_error(f"Could not get failed test IDs for run {run_id}")
            continue

        response_json = response.json()

        for test_result in response_json["value"]:
            if test_result["outcome"] == "Failed":
                failed_tests.append((run_id, test_result["id"], test_result["testCase"]["name"]))

    if not failed_tests:
        print_azure_warning("No failed tests were found. If this is because all of the tests passed, "
                            "this task should be configured to be skipped by setting the 'Run this task' option to "
                            "'Only when a previous task has failed'")

    return failed_tests


def get_file_base64(file_path):
    """
    Get the Base64 encoded string for a given file
    :param file_path: the file path to a file to be attached
    :return: the Base64 string
    """
    with open(file_path, "rb") as the_file:
        base64_string = base64.b64encode(the_file.read())

    return base64_string


def append_file_names(file_names, folder_path):
    """
    Get a list of file names inside a folder with the given path
    :param file_names: the list of file names to append to
    :param folder_path: the path to the folder to look in
    :return: 0: success (or only a warning), 1: failure
    """
    try:
        files_in_folder = listdir(folder_path)

    except FileNotFoundError:
        # The folder might not be found due to the API returning a test which didn't fail in the current deployment,
        # so this is only considered a warning
        print_azure_warning(f"Folder not found: {folder_path} - "
                            f"this could have happened due to failed tests in a previous deployment attempt")
        return 0

    if not files_in_folder:
        print_azure_error("Could not find any files in folder: " + folder_path)
        return 1

    for file in files_in_folder:
        file_names.append(file)

    return 0


def print_azure_error(error_message):
    print("##vso[task.logissue type=error]" + error_message)


def print_azure_warning(warning_message):
    print("##vso[task.logissue type=warning]" + warning_message)
